package com.nisum.offertransactionservice.controller;

import com.nisum.offertransactionservice.client.PromotionEngineFeignClient;
import com.nisum.offertransactionservice.client.StoreProducerFeignClient;
import com.nisum.offertransactionservice.dao.EndOfSaleRepo;
import com.nisum.offertransactionservice.dao.OfferLookupRepo;
import com.nisum.offertransactionservice.dao.OfferMetaDataRepo;
import com.nisum.offertransactionservice.dto.*;
import com.nisum.offertransactionservice.model.EndOfSaleEntity;
import com.nisum.offertransactionservice.model.OfferLookup;
import com.nisum.offertransactionservice.model.OfferMeta;
import com.nisum.offertransactionservice.service.OfferCallingPEService;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;
import org.junit.runner.RunWith;
import org.mockito.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Description;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;

import java.io.IOException;
import java.time.LocalDateTime;
import java.util.*;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.Mockito.when;

@SpringBootTest
@RunWith(SpringRunner.class)
class OfferTransactionControllerTest {

    @Autowired
    private OfferTransactionController offerTransactionController;

    @MockBean
    private OfferLookupRepo offerLookupRepo;

    @MockBean
    private OfferMetaDataRepo offerMetaDataRepo;

    @MockBean
    private EndOfSaleRepo endOfSaleRepo;

    @MockBean
    private PromotionEngineFeignClient clientService;

    @MockBean
    private StoreProducerFeignClient storeProducerFeignClient;

    @Autowired
    private OfferCallingPEService offerCallingPEService;


    private String HH_ID_FOR_END_OF_SALE = "id01";

    private String TRANSACTION_ID_FOR_END_OF_SALE = "tx01";

    private final String UU_ID_FOR_END_OF_TRANSACTION = "id01";

    private String DISCOUNT_DESC = "10 percent";

    @Description("Component test for getOfferTransactionResponse method")
    @Test
    void getOfferTransactionResponse() throws InterruptedException, NoSuchFieldException {

        when(offerLookupRepo.findByHhId(ArgumentMatchers.any(), ArgumentMatchers.any())).thenReturn(getOffers());
        when(clientService.callPEService(Mockito.any(PERequest.class))).thenReturn(getPeResponse());
        Mockito.doReturn(Optional.of(getOfferMeta())).when(offerMetaDataRepo).findByOfferIdAndOfferStatusIgnoreCase(Mockito.anyLong(), Mockito.anyString());
        OfferTransactionResponse offerTransactionResponse = offerTransactionController.getOfferTransactionResponse(getOfferTransactionRequest());
        assertEquals(true, Objects.nonNull(offerTransactionResponse.getTransactionId()));
    }

    @Description("Component test for endOfSale method")
    @Test
    void endOfSale() throws IOException {
        when(endOfSaleRepo.save(Mockito.any(EndOfSaleEntity.class))).thenReturn(new EndOfSaleEntity());
        assertEquals(true, Objects.nonNull(offerTransactionController.endOfSale(buildEndOfSaleReq())));
    }

    @Description("Component test for endOfTransaction method")
    @ParameterizedTest
    @ValueSource(strings = {UU_ID_FOR_END_OF_TRANSACTION})
    void endOfTransaction(String uuId) {
        when(storeProducerFeignClient.callStoreProducer(Mockito.anyString())).thenReturn(getResponseEntityString());
        assertEquals(true, Objects.nonNull(offerTransactionController.endOfTransaction(uuId)));
    }

    private List<OfferLookup> getOffers() {
        OfferLookup offerLookup1 = new OfferLookup("123", "12", 234L, "02", "AND", "BOGO");
        OfferLookup offerLookup2 = new OfferLookup("1213", "13", 234L, "02", "OR", "BXGY");
        List<OfferLookup> offerLookups = new ArrayList<>();
        offerLookups.add(offerLookup1);
        offerLookups.add(offerLookup2);
        return offerLookups;
    }

    private OfferTransactionRequest getOfferTransactionRequest() {
        OfferTransactionRequest offerTransactionRequest = new OfferTransactionRequest();
        offerTransactionRequest.setHhId("54321");
        offerTransactionRequest.setItemList(getItemList());
        return offerTransactionRequest;
    }

    private ResponseEntity<PEResponse> getPeResponse() {
        PEResponse peResponse = new PEResponse();
        peResponse.setHhid(123L);
        peResponse.setDiscountedItemList(getItemList());
        return new ResponseEntity<PEResponse>(peResponse, HttpStatus.ACCEPTED);
    }

    private EndOfSaleReq buildEndOfSaleReq() {
        OfferTransactionResponse offerTransactionResponse = new OfferTransactionResponse(HH_ID_FOR_END_OF_SALE, TRANSACTION_ID_FOR_END_OF_SALE, getItemList(), DISCOUNT_DESC);
        EndOfSaleReq endOfSaleReq = new EndOfSaleReq(offerTransactionResponse, getOffers());
        return endOfSaleReq;
    }

    private List<Item> getItemList() {
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke", "12", 13.05);
        Item item2 = new Item("Thumpsup", "11", 14.05);
        itemList.add(item1);
        itemList.add(item2);
        return itemList;
    }

    private ResponseEntity<String> getResponseEntityString() {
        return new ResponseEntity<String>("STRING", HttpStatus.ACCEPTED);
    }


    private OfferMeta getOfferMeta() {
        OfferMeta offerMetaDTO = new OfferMeta();
        offerMetaDTO.setDiscount("discount");
        offerMetaDTO.setEligibility("eligibility");
        offerMetaDTO.setExpiryTime(LocalDateTime.of(2021, 01, 01, 01, 01));
        offerMetaDTO.setStartTime(LocalDateTime.of(2020, 01, 01, 01, 01));
        offerMetaDTO.setOfferDesc("30 percent");
        offerMetaDTO.setOfferId(1L);
        offerMetaDTO.setOfferStatus("Active");
        offerMetaDTO.setRcptTxt("text");
        offerMetaDTO.setOfferType("aaa");
        return offerMetaDTO;
    }


    private List<OfferLookupDTO> getOfferLookupDto() {
        OfferLookupDTO offerLookupDTO = new OfferLookupDTO();
        offerLookupDTO.setId("123");
        offerLookupDTO.setIdType("1234");
        offerLookupDTO.setOfferId(1L);
        offerLookupDTO.setPreCondition("And");
        offerLookupDTO.setStoreId("001");
        offerLookupDTO.setTerminal("001");

        OfferMetaDTO offerMetaDTO = new OfferMetaDTO();
        offerMetaDTO.setDiscount("discount");
        offerMetaDTO.setEligibility("eligibility");
        offerMetaDTO.setExpiryTime(LocalDateTime.of(2021, 01, 01, 01, 01));
        offerMetaDTO.setStartTime(LocalDateTime.of(2020, 01, 01, 01, 01));
        offerMetaDTO.setOfferDesc("30 percent");
        offerMetaDTO.setOfferId(1L);
        offerMetaDTO.setOfferStatus("Active");
        offerMetaDTO.setRcptTxt("text");
        offerMetaDTO.setOfferType("aaa");

        offerLookupDTO.setOfferMetaDTO(offerMetaDTO);


        OfferLookupDTO offerLookupDTO1 = new OfferLookupDTO();
        offerLookupDTO1.setId("123");
        offerLookupDTO1.setIdType("1234");
        offerLookupDTO1.setOfferId(1L);
        offerLookupDTO1.setPreCondition("And");
        offerLookupDTO1.setStoreId("001");
        offerLookupDTO1.setTerminal("001");

        OfferMetaDTO offerMetaDTO1 = new OfferMetaDTO();
        offerMetaDTO1.setDiscount("discount");
        offerMetaDTO1.setEligibility("eligibility");
        offerMetaDTO1.setExpiryTime(LocalDateTime.of(2021, 01, 01, 01, 01));
        offerMetaDTO1.setStartTime(LocalDateTime.of(2020, 01, 01, 01, 01));
        offerMetaDTO1.setOfferDesc("20 percent");
        offerMetaDTO1.setOfferId(1L);
        offerMetaDTO1.setOfferStatus("Active");
        offerMetaDTO1.setRcptTxt("text");
        offerMetaDTO1.setOfferType("aaa");

        offerLookupDTO1.setOfferMetaDTO(offerMetaDTO);


        List<OfferLookupDTO> offerLookupDTOList = Arrays.asList(offerLookupDTO1, offerLookupDTO);
        return offerLookupDTOList;


    }
}